function sys = ORD2011()
    % ORD2011  O'Hara-Rudy model of human ventricular action potential.
    % The ORd model is described in the "Simulation of the Undiseased
    % Human Cardiac Ventricular Action Potential: Model Formulation and
    % Experimental Validation" by Thomas O'Hara, Laszlo Virag, Andras
    % Varro, and Yoram Rudy (2011).
    %
    % The article and supplemental materials are freely available in the
    % Open Access jounal PLoS Computational Biology
    % http://www.ploscompbiol.org/article/info:doi/10.1371/journal.pcbi.1002061
    %
    % This implementation is for the Brain Dynamics Toolbox (Version 2020)
    % http://www.bdtoolbox.org
    %
    % EXAMPLE
    % >> sys = ORD2011();
    % >> gui = bdGUI(sys);
    %
    % AUTHORS
    % Thomas O'Hara and Yoram Rudy wrote the original software for the ORd 
    % model in 2011. Stewart Heitmann ported it to the Brain Dynamics
    % Toolbox in 2018.
    %
    % LICENSE
    % The ORd model is free software: you can redistribute it and/or modify it
    % under the terms of the GNU General Public License as published by the
    % Free Software Foundation, either version 3 of the License, or (at your
    % option) any later version. The ORd model is distributed in the hope that
    % it will be useful, but WITHOUT ANY WARRANTY; without even the implied
    % warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    % GNU General Public License for more details (www.gnu.org/licenses)   

    % Handle to our ODE function
    sys.odefun = @odefun;
    
    % ODE parameter definitions
    CL = 1000;                                      % cycle length
    sys.pardef = [
        struct('name','cellType',    'value', 1,       'lim',[0 2]);
        struct('name','stimAmp',     'value',-70,      'lim',[-100 0]);    % stimulus amplitude
        struct('name','stimDur',     'value',0.5,      'lim',[0 10]);      % stimulus duration
        struct('name','GNa',         'value',75,       'lim',[0 100]);     % Na conductance
        struct('name','GNaL',        'value',0.0075,   'lim',[0 0.1]);     % NaL conductance
        struct('name','Gto',         'value',0.02,     'lim',[0 1]);       % Transient Outward conductance
        struct('name','PCa',         'value',0.0001,   'lim',[0 0.01]);    % Ca pump conductance
        struct('name','GKr',         'value',0.046,    'lim',[0 1]);       % Kr conductance
        struct('name','GKs',         'value',0.0034,   'lim',[0 0.1]);     % Ks conductance
        struct('name','GK1',         'value',0.1908,   'lim',[0 1]);       % K1 conductance
        struct('name','Gncx',        'value',0.0008,   'lim',[0 0.01]);    % NaCa_i conductance
        struct('name','Pnak',        'value',30,       'lim',[0 100]);     % NaK pump conductance
        struct('name','GKb',         'value',0.003,    'lim',[0 0.1]);     % Kb conductance
        struct('name','PNab',        'value',3.75e-10, 'lim',[0 1e-8]);    % Nab pump conductance
        struct('name','PCab',        'value',2.5e-8,   'lim',[0 1e-6]);    % Cab pump conductance
        struct('name','GpCa',        'value',0.0005,   'lim',[0 1e-2]);    % pCa conductance
        ];
    
    % ODE variable definitions
    sys.vardef = [ 
        struct('name','v',      'value',-87,  'lim',[-100 45]);
        struct('name','nai',    'value',7,    'lim',[7 8]);
        struct('name','nass',   'value',7,    'lim',[7 8]);
        struct('name','ki',     'value',145,  'lim',[140 150]);
        struct('name','kss',    'value',145,  'lim',[140 150]);
        struct('name','cai',    'value',1e-4, 'lim',[0 0.0008]);
        struct('name','cass',   'value',1e-4, 'lim',[0 0.04]);
        struct('name','cansr',  'value',1.2,  'lim',[1.5 2.5]);
        struct('name','cajsr',  'value',1.2,  'lim',[0 2]);
        struct('name','m',      'value',0,    'lim',[0 1]);
        struct('name','hf',     'value',1,    'lim',[0 1]);
        struct('name','hs',     'value',1,    'lim',[0 1]);
        struct('name','j',      'value',1,    'lim',[0 1]);
        struct('name','hsp',    'value',1,    'lim',[0 1]);
        struct('name','jp',     'value',1,    'lim',[0 1]);
        struct('name','mL',     'value',0,    'lim',[0 1]);
        struct('name','hL',     'value',1,    'lim',[0 1]);
        struct('name','hLp',    'value',1,    'lim',[0 1]);
        struct('name','a',      'value',0,    'lim',[0 1]);
        struct('name','iF',     'value',1,    'lim',[0 1]);
        struct('name','iS',     'value',1,    'lim',[0 1]);
        struct('name','ap',     'value',0,    'lim',[0 1]);
        struct('name','iFp',    'value',1,    'lim',[0 1]);
        struct('name','iSp',    'value',1,    'lim',[0 1]);
        struct('name','d',      'value',0,    'lim',[0 1]);
        struct('name','ff',     'value',1,    'lim',[0 1]);
        struct('name','fs',     'value',1,    'lim',[0 1]);
        struct('name','fcaf',   'value',1,    'lim',[0 1]);
        struct('name','fcas',   'value',1,    'lim',[0 1]);
        struct('name','jca',    'value',1,    'lim',[0 1]);
        struct('name','nca',    'value',0,    'lim',[0 1]);
        struct('name','ffp',    'value',1,    'lim',[0 1]);
        struct('name','fcafp',  'value',1,    'lim',[0 1]);
        struct('name','xrf',    'value',0,    'lim',[0 1]);
        struct('name','xrs',    'value',0,    'lim',[0 1]);
        struct('name','xs1',    'value',0,    'lim',[0 1]);
        struct('name','xs2',    'value',0,    'lim',[0 1]);
        struct('name','xk1',    'value',1,    'lim',[0 1]);
        struct('name','Jrelnp', 'value',0,    'lim',[0 2]);
        struct('name','Jrelp',  'value',0,    'lim',[0 2]);
        struct('name','CaMKt',  'value',0,    'lim',[0 0.05]);
        ];

    % Latex (Equations) panel
    sys.panels.bdLatexPanel.title = 'Equations'; 
    sys.panels.bdLatexPanel.latex = { 
        '$\textbf{ORD2011}$';
        'O''Hara-Rudy model of Human Cardiac Ventricular Action Potential.';
        '';
        '$\textbf{Parameters}$';
        '{ }{ }{ } cellType: 0=endo, 1=epi, 2=mid.';
        '{ }{ }{ } stimAmp=-70 mA   is the amplitude of the stimulus pulse';
        '{ }{ }{ } stimDur=0.5 ms   is the duration of the stimulus pulse';
        '{ }{ }{ } GNa  = 75        is the Na conductance';
        '{ }{ }{ } GNaL = 0.0075    is the late Na conductance';
        '{ }{ }{ } Gto  = 0.02      is the Transient Outward conductance';
        '{ }{ }{ } PCa  = 0.0001    is the Ca pump conductance';
        '{ }{ }{ } GKr  = 0.046     is the Kr conductance';
        '{ }{ }{ } GKs  = 0.0034    is the Ks conductance';
        '{ }{ }{ } GK1  = 0.1908    is the K1 conductance';
        '{ }{ }{ } Gncx = 0.0008    is the NaCa conductance';
        '{ }{ }{ } Pnak = 30        is the NaK pump conductance';
        '{ }{ }{ } GKb  = 0.003     is the Kb conductance';
        '{ }{ }{ } PNab = 3.75e-10  is the Nab pump conductance';
        '{ }{ }{ } PCab = 2.5e-8    is the Cab pump conductance';
        '{ }{ }{ } GPCa = 0.0005    is the pCa conductance';        
        '';
        '$\textbf{State Variables}$';
        '{ }{ }{ } v(t) is membrane potential';
        '{ }{ }{ } See O''Hara et al (2011) for the others';
        '';
        '$\textbf{Reference}$';
        'O''Hara, Virag, Varro, Rudy (2011) Simulation of the undiseased human';
        'cardiac ventricular action potential: Model formulation and experimental';
        'validation. PLoS Comput Biol 7(5): e1002061. doi:10.1371/journal.pcbi.1002061';
        };

    % Time Portrait panel 
    sys.panels.bdTimePortrait = [];

    % Auxiliary panel
    sys.panels.bdAuxiliary.auxfun = {@Stimulus,@INa,@INaL,@Ito,@ICaL,@IKr,@IKs,@IK1,@INaCa_i,@INaCa_ss,@INaK,@IKb,@INab,@ICab,@IpCa,@Jdiff,@JdiffNa,@JdiffK,@Jup,@Jleak,@Jtr,@Jrel,@CaMKa};
 
    % Solver panel
    sys.panels.bdSolverPanel = [];
    
    % Default time span (optional)
    sys.tspan = [0 CL]; 
    
    % ODE solver options (optional)
    sys.odesolver = {@ode15s, @ode23, @ode23s, @ode45};
    sys.odeoption.RelTol = 1e-6;        % Relative Tolerance
    sys.odeoption.InitialStep = 0.1;    % Solver Initial Step (should be less than StimDur)
    
    % Custom Data
    sys.UserData.Currents = @Currents;  % Custom function to reconstruct the currents
end

% ODE function from Ohara, et al.
function [output,currents] = odefun(t,X,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % force cellType into one of 0,1,2
    % where endo=0, epi=1, M=2
    celltype = floor(max(min(celltype,2),0));

    %extracellular ionic concentrations
    nao=140.0;
    cao=1.8;
    ko=5.4;

    %physical constants
    R=8314.0;
    T=310.0;
    F=96485.0;

    %cell geometry
    L=0.01;
    rad=0.0011;
    vcell=1000*3.14*rad*rad*L;
    Ageo=2*3.14*rad*rad+2*3.14*rad*L;
    Acap=2*Ageo;
    vmyo=0.68*vcell;
    vnsr=0.0552*vcell;
    vjsr=0.0048*vcell;
    vss=0.02*vcell;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %give names to the state vector values
    v=X(1);
    nai=X(2);
    nass=X(3);
    ki=X(4);
    kss=X(5);
    cai=X(6);
    cass=X(7);
    cansr=X(8);
    cajsr=X(9);
    m=X(10);
    hf=X(11);
    hs=X(12);
    j=X(13);
    hsp=X(14);
    jp=X(15);
    mL=X(16);
    hL=X(17);
    hLp=X(18);
    a=X(19);
    iF=X(20);
    iS=X(21);
    ap=X(22);
    iFp=X(23);
    iSp=X(24);
    d=X(25);
    ff=X(26);
    fs=X(27);
    fcaf=X(28);
    fcas=X(29);
    jca=X(30);
    nca=X(31);
    ffp=X(32);
    fcafp=X(33);
    xrf=X(34);
    xrs=X(35);
    xs1=X(36);
    xs2=X(37);
    xk1=X(38);
    Jrelnp=X(39);
    Jrelp=X(40);
    CaMKt=X(41);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %CaMK constants
    KmCaMK=0.15;

    aCaMK=0.05;
    bCaMK=0.00068;
    CaMKo=0.05;
    KmCaM=0.0015;
    %update CaMK
    CaMKb=CaMKo*(1.0-CaMKt)/(1.0+KmCaM/cass);
    CaMKa=CaMKb+CaMKt;
    dCaMKt=aCaMK*CaMKb*(CaMKb+CaMKt)-bCaMK*CaMKt;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %reversal potentials
    ENa=(R*T/F)*log(nao/nai);
    EK=(R*T/F)*log(ko/ki);
    PKNa=0.01833;
    EKs=(R*T/F)*log((ko+PKNa*nao)/(ki+PKNa*nai));

    %convenient shorthand calculations
    vffrt=v*F*F/(R*T);
    vfrt=v*F/(R*T);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %calculate INa
    mss=1.0/(1.0+exp((-(v+39.57))/9.871));
    tm=1.0/(6.765*exp((v+11.64)/34.77)+8.552*exp(-(v+77.42)/5.955));
    dm=(mss-m)/tm;

    hss=1.0/(1+exp((v+82.90)/6.086));
    thf=1.0/(1.432e-5*exp(-(v+1.196)/6.285)+6.149*exp((v+0.5096)/20.27));
    ths=1.0/(0.009794*exp(-(v+17.95)/28.05)+0.3343*exp((v+5.730)/56.66));
    Ahf=0.99;
    Ahs=1.0-Ahf;
    dhf=(hss-hf)/thf;
    dhs=(hss-hs)/ths;
    h=Ahf*hf+Ahs*hs;
    jss=hss;
    tj=2.038+1.0/(0.02136*exp(-(v+100.6)/8.281)+0.3052*exp((v+0.9941)/38.45));
    dj=(jss-j)/tj;
    hssp=1.0/(1+exp((v+89.1)/6.086));
    thsp=3.0*ths;
    dhsp=(hssp-hsp)/thsp;
    hp=Ahf*hf+Ahs*hsp;
    tjp=1.46*tj;
    djp=(jss-jp)/tjp;
    %GNa=75;
    fINap=(1.0/(1.0+KmCaMK/CaMKa));
    INa=GNa*(v-ENa)*m^3.0*((1.0-fINap)*h*j+fINap*hp*jp);

    %calculate INaL
    mLss=1.0/(1.0+exp((-(v+42.85))/5.264));
    tmL=tm;
    dmL=(mLss-mL)/tmL;
    hLss=1.0/(1.0+exp((v+87.61)/7.488));
    thL=200.0;
    dhL=(hLss-hL)/thL;
    hLssp=1.0/(1.0+exp((v+93.81)/7.488));
    thLp=3.0*thL;
    dhLp=(hLssp-hLp)/thLp;
    %GNaL=0.0075;
    if celltype==1
        GNaL=GNaL*0.6;
    end
    fINaLp=(1.0/(1.0+KmCaMK/CaMKa));
    INaL=GNaL*(v-ENa)*mL*((1.0-fINaLp)*hL+fINaLp*hLp);

    %calculate Ito
    ass=1.0/(1.0+exp((-(v-14.34))/14.82));
    ta=1.0515/(1.0/(1.2089*(1.0+exp(-(v-18.4099)/29.3814)))+3.5/(1.0+exp((v+100.0)/29.3814)));
    da=(ass-a)/ta;
    iss=1.0/(1.0+exp((v+43.94)/5.711));
    if celltype==1
        delta_epi=1.0-(0.95/(1.0+exp((v+70.0)/5.0)));
    else
        delta_epi=1.0;
    end
    tiF=4.562+1/(0.3933*exp((-(v+100.0))/100.0)+0.08004*exp((v+50.0)/16.59));
    tiS=23.62+1/(0.001416*exp((-(v+96.52))/59.05)+1.780e-8*exp((v+114.1)/8.079));
    tiF=tiF*delta_epi;
    tiS=tiS*delta_epi;
    AiF=1.0/(1.0+exp((v-213.6)/151.2));
    AiS=1.0-AiF;
    diF=(iss-iF)/tiF;
    diS=(iss-iS)/tiS;
    i=AiF*iF+AiS*iS;
    assp=1.0/(1.0+exp((-(v-24.34))/14.82));
    dap=(assp-ap)/ta;
    dti_develop=1.354+1.0e-4/(exp((v-167.4)/15.89)+exp(-(v-12.23)/0.2154));
    dti_recover=1.0-0.5/(1.0+exp((v+70.0)/20.0));
    tiFp=dti_develop*dti_recover*tiF;
    tiSp=dti_develop*dti_recover*tiS;
    diFp=(iss-iFp)/tiFp;
    diSp=(iss-iSp)/tiSp;
    ip=AiF*iFp+AiS*iSp;
    %Gto=0.02;
    if celltype==1
        Gto=Gto*4.0;
    elseif celltype==2
        Gto=Gto*4.0;
    end
    fItop=(1.0/(1.0+KmCaMK/CaMKa));
    Ito=Gto*(v-EK)*((1.0-fItop)*a*i+fItop*ap*ip);

    %calculate ICaL, ICaNa, ICaK
    dss=1.0/(1.0+exp((-(v+3.940))/4.230));
    td=0.6+1.0/(exp(-0.05*(v+6.0))+exp(0.09*(v+14.0)));
    dd=(dss-d)/td;
    fss=1.0/(1.0+exp((v+19.58)/3.696));
    tff=7.0+1.0/(0.0045*exp(-(v+20.0)/10.0)+0.0045*exp((v+20.0)/10.0));
    tfs=1000.0+1.0/(0.000035*exp(-(v+5.0)/4.0)+0.000035*exp((v+5.0)/6.0));
    Aff=0.6;
    Afs=1.0-Aff;
    dff=(fss-ff)/tff;
    dfs=(fss-fs)/tfs;
    f=Aff*ff+Afs*fs;
    fcass=fss;
    tfcaf=7.0+1.0/(0.04*exp(-(v-4.0)/7.0)+0.04*exp((v-4.0)/7.0));
    tfcas=100.0+1.0/(0.00012*exp(-v/3.0)+0.00012*exp(v/7.0));
    Afcaf=0.3+0.6/(1.0+exp((v-10.0)/10.0));
    Afcas=1.0-Afcaf;
    dfcaf=(fcass-fcaf)/tfcaf;
    dfcas=(fcass-fcas)/tfcas;
    fca=Afcaf*fcaf+Afcas*fcas;
    tjca=75.0;
    djca=(fcass-jca)/tjca;
    tffp=2.5*tff;
    dffp=(fss-ffp)/tffp;
    fp=Aff*ffp+Afs*fs;
    tfcafp=2.5*tfcaf;
    dfcafp=(fcass-fcafp)/tfcafp;
    fcap=Afcaf*fcafp+Afcas*fcas;
    Kmn=0.002;
    k2n=1000.0;
    km2n=jca*1.0;
    anca=1.0/(k2n/km2n+(1.0+Kmn/cass)^4.0);
    dnca=anca*k2n-nca*km2n;
    PhiCaL=4.0*vffrt*(cass*exp(2.0*vfrt)-0.341*cao)/(exp(2.0*vfrt)-1.0);
    PhiCaNa=1.0*vffrt*(0.75*nass*exp(1.0*vfrt)-0.75*nao)/(exp(1.0*vfrt)-1.0);
    PhiCaK=1.0*vffrt*(0.75*kss*exp(1.0*vfrt)-0.75*ko)/(exp(1.0*vfrt)-1.0);
    zca=2.0;
    %PCa=0.0001;
    if celltype==1
        PCa=PCa*1.2;
    elseif celltype==2
        PCa=PCa*2.5;
    end
    PCap=1.1*PCa;
    PCaNa=0.00125*PCa;
    PCaK=3.574e-4*PCa;
    PCaNap=0.00125*PCap;
    PCaKp=3.574e-4*PCap;
    fICaLp=(1.0/(1.0+KmCaMK/CaMKa));
    ICaL=((1.0-fICaLp)*PCa*PhiCaL*d*(f*(1.0-nca)+jca*fca*nca)+fICaLp*PCap*PhiCaL*d*(fp*(1.0-nca)+jca*fcap*nca));
    ICaNa=(1.0-fICaLp)*PCaNa*PhiCaNa*d*(f*(1.0-nca)+jca*fca*nca)+fICaLp*PCaNap*PhiCaNa*d*(fp*(1.0-nca)+jca*fcap*nca);
    ICaK=(1.0-fICaLp)*PCaK*PhiCaK*d*(f*(1.0-nca)+jca*fca*nca)+fICaLp*PCaKp*PhiCaK*d*(fp*(1.0-nca)+jca*fcap*nca);

    %calculate IKr
    xrss=1.0/(1.0+exp((-(v+8.337))/6.789));
    txrf=12.98+1.0/(0.3652*exp((v-31.66)/3.869)+4.123e-5*exp((-(v-47.78))/20.38));
    txrs=1.865+1.0/(0.06629*exp((v-34.70)/7.355)+1.128e-5*exp((-(v-29.74))/25.94));
    Axrf=1.0/(1.0+exp((v+54.81)/38.21));
    Axrs=1.0-Axrf;
    dxrf=(xrss-xrf)/txrf;
    dxrs=(xrss-xrs)/txrs;
    xr=Axrf*xrf+Axrs*xrs;
    rkr=1.0/(1.0+exp((v+55.0)/75.0))*1.0/(1.0+exp((v-10.0)/30.0));
    %GKr=0.046;
    if celltype==1
        GKr=GKr*1.3;
    elseif celltype==2
        GKr=GKr*0.8;
    end
    IKr=GKr*sqrt(ko/5.4)*xr*rkr*(v-EK);

    %calculate IKs
    xs1ss=1.0/(1.0+exp((-(v+11.60))/8.932));
    txs1=817.3+1.0/(2.326e-4*exp((v+48.28)/17.80)+0.001292*exp((-(v+210.0))/230.0));
    dxs1=(xs1ss-xs1)/txs1;
    xs2ss=xs1ss;
    txs2=1.0/(0.01*exp((v-50.0)/20.0)+0.0193*exp((-(v+66.54))/31.0));
    dxs2=(xs2ss-xs2)/txs2;
    KsCa=1.0+0.6/(1.0+(3.8e-5/cai)^1.4);
    %GKs=0.0034;
    if celltype==1
        GKs=GKs*1.4;
    end
    IKs=GKs*KsCa*xs1*xs2*(v-EKs);

    xk1ss=1.0/(1.0+exp(-(v+2.5538*ko+144.59)/(1.5692*ko+3.8115)));
    txk1=122.2/(exp((-(v+127.2))/20.36)+exp((v+236.8)/69.33));
    dxk1=(xk1ss-xk1)/txk1;
    rk1=1.0/(1.0+exp((v+105.8-2.6*ko)/9.493));
    %GK1=0.1908;
    if celltype==1
        GK1=GK1*1.2;
    elseif celltype==2
        GK1=GK1*1.3;
    end
    IK1=GK1*sqrt(ko)*rk1*xk1*(v-EK);

    %calculate INaCa_i
    kna1=15.0;
    kna2=5.0;
    kna3=88.12;
    kasymm=12.5;
    wna=6.0e4;
    wca=6.0e4;
    wnaca=5.0e3;
    kcaon=1.5e6;
    kcaoff=5.0e3;
    qna=0.5224;
    qca=0.1670;
    hca=exp((qca*v*F)/(R*T));
    hna=exp((qna*v*F)/(R*T));
    h1=1+nai/kna3*(1+hna);
    h2=(nai*hna)/(kna3*h1);
    h3=1.0/h1;
    h4=1.0+nai/kna1*(1+nai/kna2);
    h5=nai*nai/(h4*kna1*kna2);
    h6=1.0/h4;
    h7=1.0+nao/kna3*(1.0+1.0/hna);
    h8=nao/(kna3*hna*h7);
    h9=1.0/h7;
    h10=kasymm+1.0+nao/kna1*(1.0+nao/kna2);
    h11=nao*nao/(h10*kna1*kna2);
    h12=1.0/h10;
    k1=h12*cao*kcaon;
    k2=kcaoff;
    k3p=h9*wca;
    k3pp=h8*wnaca;
    k3=k3p+k3pp;
    k4p=h3*wca/hca;
    k4pp=h2*wnaca;
    k4=k4p+k4pp;
    k5=kcaoff;
    k6=h6*cai*kcaon;
    k7=h5*h2*wna;
    k8=h8*h11*wna;
    x1=k2*k4*(k7+k6)+k5*k7*(k2+k3);
    x2=k1*k7*(k4+k5)+k4*k6*(k1+k8);
    x3=k1*k3*(k7+k6)+k8*k6*(k2+k3);
    x4=k2*k8*(k4+k5)+k3*k5*(k1+k8);
    E1=x1/(x1+x2+x3+x4);
    E2=x2/(x1+x2+x3+x4);
    E3=x3/(x1+x2+x3+x4);
    E4=x4/(x1+x2+x3+x4);
    KmCaAct=150.0e-6;
    allo=1.0/(1.0+(KmCaAct/cai)^2.0);
    zna=1.0;
    JncxNa=3.0*(E4*k7-E1*k8)+E3*k4pp-E2*k3pp;
    JncxCa=E2*k2-E1*k1;
    %Gncx=0.0008;
    if celltype==1
        Gncx=Gncx*1.1;
    elseif celltype==2
        Gncx=Gncx*1.4;
    end
    INaCa_i=0.8*Gncx*allo*(zna*JncxNa+zca*JncxCa);

    %calculate INaCa_ss
    h1=1+nass/kna3*(1+hna);
    h2=(nass*hna)/(kna3*h1);
    h3=1.0/h1;
    h4=1.0+nass/kna1*(1+nass/kna2);
    h5=nass*nass/(h4*kna1*kna2);
    h6=1.0/h4;
    h7=1.0+nao/kna3*(1.0+1.0/hna);
    h8=nao/(kna3*hna*h7);
    h9=1.0/h7;
    h10=kasymm+1.0+nao/kna1*(1+nao/kna2);
    h11=nao*nao/(h10*kna1*kna2);
    h12=1.0/h10;
    k1=h12*cao*kcaon;
    k2=kcaoff;
    k3p=h9*wca;
    k3pp=h8*wnaca;
    k3=k3p+k3pp;
    k4p=h3*wca/hca;
    k4pp=h2*wnaca;
    k4=k4p+k4pp;
    k5=kcaoff;
    k6=h6*cass*kcaon;
    k7=h5*h2*wna;
    k8=h8*h11*wna;
    x1=k2*k4*(k7+k6)+k5*k7*(k2+k3);
    x2=k1*k7*(k4+k5)+k4*k6*(k1+k8);
    x3=k1*k3*(k7+k6)+k8*k6*(k2+k3);
    x4=k2*k8*(k4+k5)+k3*k5*(k1+k8);
    E1=x1/(x1+x2+x3+x4);
    E2=x2/(x1+x2+x3+x4);
    E3=x3/(x1+x2+x3+x4);
    E4=x4/(x1+x2+x3+x4);
    KmCaAct=150.0e-6;
    allo=1.0/(1.0+(KmCaAct/cass)^2.0);
    JncxNa=3.0*(E4*k7-E1*k8)+E3*k4pp-E2*k3pp;
    JncxCa=E2*k2-E1*k1;
    INaCa_ss=0.2*Gncx*allo*(zna*JncxNa+zca*JncxCa);

    %calculate INaK
    k1p=949.5;
    k1m=182.4;
    k2p=687.2;
    k2m=39.4;
    k3p=1899.0;
    k3m=79300.0;
    k4p=639.0;
    k4m=40.0;
    Knai0=9.073;
    Knao0=27.78;
    delta=-0.1550;
    Knai=Knai0*exp((delta*v*F)/(3.0*R*T));
    Knao=Knao0*exp(((1.0-delta)*v*F)/(3.0*R*T));
    Kki=0.5;
    Kko=0.3582;
    MgADP=0.05;
    MgATP=9.8;
    Kmgatp=1.698e-7;
    H=1.0e-7;
    eP=4.2;
    Khp=1.698e-7;
    Knap=224.0;
    Kxkur=292.0;
    P=eP/(1.0+H/Khp+nai/Knap+ki/Kxkur);
    a1=(k1p*(nai/Knai)^3.0)/((1.0+nai/Knai)^3.0+(1.0+ki/Kki)^2.0-1.0);
    b1=k1m*MgADP;
    a2=k2p;
    b2=(k2m*(nao/Knao)^3.0)/((1.0+nao/Knao)^3.0+(1.0+ko/Kko)^2.0-1.0);
    a3=(k3p*(ko/Kko)^2.0)/((1.0+nao/Knao)^3.0+(1.0+ko/Kko)^2.0-1.0);
    b3=(k3m*P*H)/(1.0+MgATP/Kmgatp);
    a4=(k4p*MgATP/Kmgatp)/(1.0+MgATP/Kmgatp);
    b4=(k4m*(ki/Kki)^2.0)/((1.0+nai/Knai)^3.0+(1.0+ki/Kki)^2.0-1.0);
    x1=a4*a1*a2+b2*b4*b3+a2*b4*b3+b3*a1*a2;
    x2=b2*b1*b4+a1*a2*a3+a3*b1*b4+a2*a3*b4;
    x3=a2*a3*a4+b3*b2*b1+b2*b1*a4+a3*a4*b1;
    x4=b4*b3*b2+a3*a4*a1+b2*a4*a1+b3*b2*a1;
    E1=x1/(x1+x2+x3+x4);
    E2=x2/(x1+x2+x3+x4);
    E3=x3/(x1+x2+x3+x4);
    E4=x4/(x1+x2+x3+x4);
    zk=1.0;
    JnakNa=3.0*(E1*a3-E2*b3);
    JnakK=2.0*(E4*b1-E3*a1);
    %Pnak=30;
    if celltype==1
        Pnak=Pnak*0.9;
    elseif celltype==2
        Pnak=Pnak*0.7;
    end
    INaK=Pnak*(zna*JnakNa+zk*JnakK);

    %calculate IKb
    xkb=1.0/(1.0+exp(-(v-14.48)/18.34));
    %GKb=0.003;
    if celltype==1
        GKb=GKb*0.6;
    end
    IKb=GKb*xkb*(v-EK);

    %calculate INab
    %PNab=3.75e-10;
    INab=PNab*vffrt*(nai*exp(vfrt)-nao)/(exp(vfrt)-1.0);

    %calculate ICab
    %PCab=2.5e-8;
    ICab=PCab*4.0*vffrt*(cai*exp(2.0*vfrt)-0.341*cao)/(exp(2.0*vfrt)-1.0);

    %calculate IpCa
    %GpCa=0.0005;
    IpCa=GpCa*cai/(0.0005+cai);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %calculate the stimulus current, Istim
    if 0<=t && t<=stimDur
        Istim = stimAmp;
    else
        Istim=0.0;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %udtate the membrane voltage
    dv=-(INa+INaL+Ito+ICaL+ICaNa+ICaK+IKr+IKs+IK1+INaCa_i+INaCa_ss+INaK+INab+IKb+IpCa+ICab+Istim);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %calculate diffusion fluxes
    JdiffNa=(nass-nai)/2.0;
    JdiffK=(kss-ki)/2.0;
    Jdiff=(cass-cai)/0.2;

    %calculate ryanodione receptor calcium induced calcium release from the jsr
    bt=4.75;
    a_rel=0.5*bt;
    Jrel_inf=a_rel*(-ICaL)/(1.0+(1.5/cajsr)^8.0);
    if celltype==2
        Jrel_inf=Jrel_inf*1.7;
    end
    tau_rel=bt/(1.0+0.0123/cajsr);

    if tau_rel<0.001
       tau_rel=0.001; 
    end

    dJrelnp=(Jrel_inf-Jrelnp)/tau_rel;
    btp=1.25*bt;
    a_relp=0.5*btp;
    Jrel_infp=a_relp*(-ICaL)/(1.0+(1.5/cajsr)^8.0);
    if celltype==2
        Jrel_infp=Jrel_infp*1.7;
    end
    tau_relp=btp/(1.0+0.0123/cajsr);

    if tau_relp<0.001
       tau_relp=0.001; 
    end

    dJrelp=(Jrel_infp-Jrelp)/tau_relp;
    fJrelp=(1.0/(1.0+KmCaMK/CaMKa));
    Jrel=(1.0-fJrelp)*Jrelnp+fJrelp*Jrelp;

    %calculate serca pump, ca uptake flux
    Jupnp=0.004375*cai/(cai+0.00092);
    Jupp=2.75*0.004375*cai/(cai+0.00092-0.00017);
    if celltype==1
        Jupnp=Jupnp*1.3;
        Jupp=Jupp*1.3;
    end
    fJupp=(1.0/(1.0+KmCaMK/CaMKa));
    Jleak=0.0039375*cansr/15.0;
    Jup=(1.0-fJupp)*Jupnp+fJupp*Jupp-Jleak;

    %calculate tranlocation flux
    Jtr=(cansr-cajsr)/100.0;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %calcium buffer constants
    cmdnmax=0.05;
    if celltype==1
        cmdnmax=cmdnmax*1.3;
    end
    kmcmdn=0.00238;
    trpnmax=0.07;
    kmtrpn=0.0005;
    BSRmax=0.047;
    KmBSR=0.00087;
    BSLmax=1.124;
    KmBSL=0.0087;
    csqnmax=10.0;
    kmcsqn=0.8;

    %update intracellular concentrations, using buffers for cai, cass, cajsr
    dnai=-(INa+INaL+3.0*INaCa_i+3.0*INaK+INab)*Acap/(F*vmyo)+JdiffNa*vss/vmyo;
    dnass=-(ICaNa+3.0*INaCa_ss)*Acap/(F*vss)-JdiffNa;

    dki=-(Ito+IKr+IKs+IK1+IKb+Istim-2.0*INaK)*Acap/(F*vmyo)+JdiffK*vss/vmyo;
    dkss=-(ICaK)*Acap/(F*vss)-JdiffK;

    Bcai=1.0/(1.0+cmdnmax*kmcmdn/(kmcmdn+cai)^2.0+trpnmax*kmtrpn/(kmtrpn+cai)^2.0);
    dcai=Bcai*(-(IpCa+ICab-2.0*INaCa_i)*Acap/(2.0*F*vmyo)-Jup*vnsr/vmyo+Jdiff*vss/vmyo);

    Bcass=1.0/(1.0+BSRmax*KmBSR/(KmBSR+cass)^2.0+BSLmax*KmBSL/(KmBSL+cass)^2.0);
    dcass=Bcass*(-(ICaL-2.0*INaCa_ss)*Acap/(2.0*F*vss)+Jrel*vjsr/vss-Jdiff);

    dcansr=Jup-Jtr*vjsr/vnsr;

    Bcajsr=1.0/(1.0+csqnmax*kmcsqn/(kmcsqn+cajsr)^2.0);
    dcajsr=Bcajsr*(Jtr-Jrel);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %output the state vector
    output=[dv dnai dnass dki dkss dcai dcass dcansr dcajsr dm dhf dhs dj dhsp djp dmL dhL dhLp da diF diS dap diFp diSp dd dff dfs dfcaf dfcas djca dnca dffp dfcafp dxrf dxrs dxs1 dxs2 dxk1 dJrelnp dJrelp dCaMKt]';

    % output the currents
    if nargout>1
        currents=[INa INaL Ito ICaL IKr IKs IK1 INaCa_i INaCa_ss INaK  IKb INab ICab IpCa Jdiff JdiffNa JdiffK Jup Jleak Jtr Jrel CaMKa Istim];
    end
end


% Auxiliary function for plotting the profile of the stimulus 
function auxdata = Stimulus(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the stimulus
    stairs(ax,auxdata.t, auxdata.Istim, 'color','k', 'LineWidth',1);
    ylabel(ax,'Istim');
    xlabel(ax,'time');
    title(ax,'Stimulus Pulse Profile');
end

% Auxiliary function for plotting INa 
function auxdata = INa(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the INa current
    plot(ax,auxdata.t, auxdata.INa);
    xlabel(ax,'time');
    ylabel(ax,'INa');
    title(ax,'INa Current Profile');
end

% Auxiliary function for plotting INa 
function auxdata = INaL(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the INaL current
    plot(ax,auxdata.t, auxdata.INaL);
    xlabel(ax,'time');
    ylabel(ax,'INaL');
    title(ax,'INaL Current Profile');
end

% Auxiliary function for plotting Ito
function auxdata = Ito(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the Ito current
    plot(ax,auxdata.t, auxdata.Ito);
    xlabel(ax,'time');
    ylabel(ax,'Ito');
    title(ax,'Ito Current Profile');
end

% Auxiliary function for plotting ICaL
function auxdata = ICaL(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the ICaL current
    plot(ax,auxdata.t, auxdata.ICaL);
    xlabel(ax,'time');
    ylabel(ax,'ICaL');
    title(ax,'ICal Current Profile');
end

% Auxiliary function for plotting IKr 
function auxdata = IKr(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the IKr current
    plot(ax,auxdata.t, auxdata.IKr);
    xlabel(ax,'time');
    ylabel(ax,'IKr');
    title(ax,'IKr Current Profile');
end

% Auxiliary function for plotting IKs 
function auxdata = IKs(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the IKs current
    plot(ax,auxdata.t, auxdata.IKs);
    xlabel(ax,'time');
    ylabel(ax,'IKs');
    title(ax,'IKs Current Profile');
end

% Auxiliary function for plotting IK1
function auxdata = IK1(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the IK1 current
    plot(ax,auxdata.t, auxdata.IK1);
    xlabel(ax,'time');
    ylabel(ax,'IK1');
    title(ax,'IK1 Current Profile');
end

% Auxiliary function for plotting INa
function auxdata = INaCa_i(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the INaCa_i current
    plot(ax,auxdata.t, auxdata.INaCa_i);
    xlabel(ax,'time');
    ylabel(ax,'INaCa_i');
    title(ax,'INaCa_i Current Profile');
end

% Auxiliary function for plotting INaCa_ss
function auxdata = INaCa_ss(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the INa current
    plot(ax,auxdata.t, auxdata.INaCa_ss);
    xlabel(ax,'time');
    ylabel(ax,'INaCa_ss');
    title(ax,'INaCa_ss Current Profile');
end

% Auxiliary function for plotting INaK 
function auxdata = INaK(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the INaK current
    plot(ax,auxdata.t, auxdata.INaK);
    xlabel(ax,'time');
    ylabel(ax,'INaK');
    title(ax,'INaK Current Profile');
end

% Auxiliary function for plotting IKb
function auxdata = IKb(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the IKb current
    plot(ax,auxdata.t, auxdata.IKb);
    xlabel(ax,'time');
    ylabel(ax,'IKb');
    title(ax,'IKb Current Profile');
end

% Auxiliary function for plotting INab 
function auxdata = INab(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the INab current
    plot(ax,auxdata.t, auxdata.INab);
    xlabel(ax,'time');
    ylabel(ax,'INab');
    title(ax,'INab Current Profile');
end

% Auxiliary function for plotting ICab 
function auxdata = ICab(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the ICab current
    plot(ax,auxdata.t, auxdata.ICab);
    xlabel(ax,'time');
    ylabel(ax,'ICab');
    title(ax,'ICab Current Profile');
end

% Auxiliary function for plotting IpCa 
function auxdata = IpCa(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the IpCa current
    plot(ax,auxdata.t, auxdata.IpCa);
    xlabel(ax,'time');
    ylabel(ax,'IpCa');
    title(ax,'IpCa Current Profile');
end

% Auxiliary function for plotting Jdiff 
function auxdata = Jdiff(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the Jdiff current
    plot(ax,auxdata.t, auxdata.Jdiff);
    xlabel(ax,'time');
    ylabel(ax,'Jdiff');
    title(ax,'Jdiff Current Profile');
end

% Auxiliary function for plotting JdiffNa
function auxdata = JdiffNa(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the JdiffNa current
    plot(ax,auxdata.t, auxdata.JdiffNa);
    xlabel(ax,'time');
    ylabel(ax,'JdiffNa');
    title(ax,'JdiffNa Current Profile');
end

% Auxiliary function for plotting JdiffK
function auxdata = JdiffK(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the JdiffK current
    plot(ax,auxdata.t, auxdata.JdiffK);
    xlabel(ax,'time');
    ylabel(ax,'JdiffK');
    title(ax,'JdiffK Current Profile');
end

% Auxiliary function for plotting Jup 
function auxdata = Jup(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the Jup current
    plot(ax,auxdata.t, auxdata.Jup);
    xlabel(ax,'time');
    ylabel(ax,'Jup');
    title(ax,'Jup Current Profile');
end

% Auxiliary function for plotting Jleak 
function auxdata = Jleak(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the Jleak current
    plot(ax,auxdata.t, auxdata.Jleak);
    xlabel(ax,'time');
    ylabel(ax,'Jleak');
    title(ax,'Jleak Current Profile');
end

% Auxiliary function for plotting Jtr 
function auxdata = Jtr(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the Jtr current
    plot(ax,auxdata.t, auxdata.Jtr);
    xlabel(ax,'time');
    ylabel(ax,'Jtr');
    title(ax,'Jtr Current Profile');
end

% Auxiliary function for plotting Jrel 
function auxdata = Jrel(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the Jrel current
    plot(ax,auxdata.t, auxdata.Jrel);
    xlabel(ax,'time');
    ylabel(ax,'Jrel');
    title(ax,'Jrel Current Profile');
end

% Auxiliary function for plotting CaMKa 
function auxdata = CaMKa(ax,t,sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % reconstruct all currents (including the stimulus)
    auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);

    % plot the CaMKa current
    plot(ax,auxdata.t, auxdata.CaMKa);
    xlabel(ax,'time');
    ylabel(ax,'CaMKa');
    title(ax,'CaMKa Current Profile');
end

% Reconstruct all currents (including the stimulus) from the given sol. 
function auxdata = Currents(sol,celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa)
    % time steps of the solution
    auxdata.t        = sol.x;
    auxdata.V        = sol.y(1,:);
    auxdata.INa      = nan(size(sol.x));
    auxdata.INaL     = nan(size(sol.x));
    auxdata.Ito      = nan(size(sol.x));
    auxdata.ICaL     = nan(size(sol.x));
    auxdata.IKr      = nan(size(sol.x));
    auxdata.IKs      = nan(size(sol.x));
    auxdata.IK1      = nan(size(sol.x));
    auxdata.INaCa_i  = nan(size(sol.x));
    auxdata.INaCa_ss = nan(size(sol.x));
    auxdata.INaK     = nan(size(sol.x));
    auxdata.IKb      = nan(size(sol.x));
    auxdata.INab     = nan(size(sol.x));
    auxdata.ICab     = nan(size(sol.x));
    auxdata.IpCa     = nan(size(sol.x));
    auxdata.Jdiff    = nan(size(sol.x));
    auxdata.JdiffNa  = nan(size(sol.x));
    auxdata.JdiffK   = nan(size(sol.x));
    auxdata.Jup      = nan(size(sol.x));
    auxdata.Jleak    = nan(size(sol.x));
    auxdata.Jtr      = nan(size(sol.x));
    auxdata.Jrel     = nan(size(sol.x));
    auxdata.CaMKa    = nan(size(sol.x));
    auxdata.Istim    = nan(size(sol.x));
    
    % reconstruct the currents
    for indx = 1:numel(auxdata.t)
        [~,currents] = odefun(sol.x(indx),sol.y(:,indx),celltype,stimAmp,stimDur,GNa,GNaL,Gto,PCa,GKr,GKs,GK1,Gncx,Pnak,GKb,PNab,PCab,GpCa);
        auxdata.INa(indx)      = currents(1);
        auxdata.INaL(indx)     = currents(2);
        auxdata.Ito(indx)      = currents(3);
        auxdata.ICaL(indx)     = currents(4);
        auxdata.IKr(indx)      = currents(5);
        auxdata.IKs(indx)      = currents(6);
        auxdata.IK1(indx)      = currents(7);
        auxdata.INaCa_i(indx)  = currents(8);
        auxdata.INaCa_ss(indx) = currents(9);
        auxdata.INaK(indx)     = currents(10);
        auxdata.IKb(indx)      = currents(11);
        auxdata.INab(indx)     = currents(12);
        auxdata.ICab(indx)     = currents(13);
        auxdata.IpCa(indx)     = currents(14);
        auxdata.Jdiff(indx)    = currents(15);
        auxdata.JdiffNa(indx)  = currents(16);
        auxdata.JdiffK(indx)   = currents(17);
        auxdata.Jup(indx)      = currents(18);
        auxdata.Jleak(indx)    = currents(19);
        auxdata.Jtr(indx)      = currents(20);
        auxdata.Jrel(indx)     = currents(21);
        auxdata.CaMKa(indx)    = currents(22);
        auxdata.Istim(indx)    = currents(23);
    end
end
